package nl.kallestruik.dtweaks.fakeplayer

import com.mojang.authlib.GameProfile
import net.minecraft.network.chat.TextComponent
import net.minecraft.network.protocol.PacketFlow
import net.minecraft.network.protocol.game.ClientboundRotateHeadPacket
import net.minecraft.network.protocol.game.ClientboundTeleportEntityPacket
import net.minecraft.server.MinecraftServer
import net.minecraft.server.TickTask
import net.minecraft.server.level.ServerLevel
import net.minecraft.server.level.ServerPlayer
import net.minecraft.world.damagesource.DamageSource
import net.minecraft.world.entity.Entity
import net.minecraft.world.food.FoodData
import net.minecraft.world.level.block.entity.SkullBlockEntity
import org.bukkit.Location
import org.bukkit.craftbukkit.v1_18_R1.CraftWorld
import java.util.concurrent.atomic.AtomicReference

class FakePlayer(
    server: MinecraftServer,
    val serverLevel: ServerLevel,
    gameProfile: GameProfile
): ServerPlayer(server, serverLevel, gameProfile) {
    val locale = "en_US"

    companion object {
        fun atLocation(location: Location, name: String): FakePlayer? {
            // Get the minecraft server instance.
            val minecraftServer = MinecraftServer.getServer()
            // Get the world
            val serverLevel = (location.world as CraftWorld).handle
            // Get the game profile from the cache (or retrieve it if it is not cached)
            var gameProfile = minecraftServer.profileCache.get(name).takeIf { it.isPresent }?.get() ?: return null
            if (gameProfile.properties.containsKey("textures")) {
                val result = AtomicReference<GameProfile>()
                SkullBlockEntity.updateGameprofile(gameProfile, result::set)
                gameProfile = result.get()
            }

            // Create an instance of our fake player.
            val instance = FakePlayer(minecraftServer, serverLevel, gameProfile)

            // Create a fake connection for our fake player.
            val connection = FakeConnection(PacketFlow.SERVERBOUND)
            FakePlayerList.placeNewPlayer(minecraftServer.playerList, minecraftServer, connection, instance)

            instance.respawn()
            instance.teleportTo(serverLevel, location.x, location.y, location.z, location.yaw, location.pitch)
            instance.dead = false
            instance.heal(instance.maxHealth)
            instance.unsetRemoved()
            instance.maxUpStep = 0.6F

            minecraftServer.playerList.broadcastAll(
                ClientboundRotateHeadPacket(
                    instance,
                    (instance.yHeadRot * 256 / 360).toInt().toByte()
                ), serverLevel.dimension())

            minecraftServer.playerList.broadcastAll(ClientboundTeleportEntityPacket(instance), serverLevel.dimension())

            instance.entityData.set(DATA_PLAYER_MODE_CUSTOMISATION, 0x7f)
            return instance

//                (DTweaks.reflectionUtil.getValueFromField(
//                    serverConnection!!,
//                    "connectedChannels"
//                ) as MutableList<NetworkManager?>).add(connection)
//
//                // Connect the fake player to the server using the fake connection.
//                FakePlayerList.a(minecraftServer.playerList, connection, instance)
//
//                // Check if the fake player is not yet in the correct world.
//                if (instance.world.dimensionKey != worldServer.dimensionKey) {
//                    // Store the old world of the fake player.
//                    val old_world = instance.world as WorldServer
//                    // Remove the fake player from the old world.
//                    old_world.removePlayer(instance)
//                    // Make the fake player not be dead.
//                    instance.dead = false
//                    // Create the fake player in the new world.
//                    worldServer.addEntity(instance)
//                    // Spawn the fake player in the new world.
//                    instance.spawnIn(worldServer)
//                    // Move the fake player to the new world for the server.
//                    minecraftServer.playerList.moveToWorld(instance, old_world, true, null, false)
//                    // requestTeleport(x, y, z, yaw, pitch)
//                    // Request the teleport from the fake player.
//                    instance.playerConnection.a(x, y, z, yaw.toFloat(), pitch.toFloat())
//                    // Set the fake player's world to the new world.
//                    instance.playerInteractManager.world = worldServer
//                    instance.teleportTo(worldServer, BlockPosition(x, y, z))
//                }
//
//                /// Set the fake players health to max.
//                instance.health = 20.0f
//                // Make the fake player not be dead.
//                instance.dead = false
//                // a == requestTeleport
//                // Request the teleport from the fake player.
//                instance.playerConnection.a(x, y, z, yaw.toFloat(), pitch.toFloat())
//                // G == stepHeight
//                // Set the fake players step height to 0.6 (The normal value).
//                instance.G = 0.6f
//                // Set the fake players gamemode to survival.
//                playerInteractManager.gameMode = EnumGamemode.SURVIVAL
//                // Tell everyone in the world about the fake player and where he is.
//                minecraftServer.playerList.a(
//                    PacketPlayOutEntityHeadRotation(instance, (instance.yaw * 256 / 360).toByte()),
//                    instance.world.dimensionKey
//                )
//                minecraftServer.playerList.a(PacketPlayOutEntityTeleport(instance), instance.world.dimensionKey)
//                // Move the fake player for the worlds chunk provider.
//                instance.worldServer.getChunkProvider().movePlayer(instance)
//                // bp == PLAYER_MODEL_PARTS
//                instance.datawatcher.set<Byte>(bj, 0x7f.toByte()) // show all model layers (incl. capes)
        }
    }

    override fun kill() {
        server.tell(TickTask(server.tickCount) {connection.onDisconnect(TextComponent("Killed"))})
    }

    override fun die(source: DamageSource) {
        super.die(source)
        heal(maxHealth)
        foodData = FoodData(this)
        kill()
    }

    override fun tick() {
        super.tick()

        if (server.tickCount % 10 == 0) {
            connection.resetPosition()
            serverLevel.chunkSource.move(this)
            hasChangedDimension()
        }
    }

//    override fun die(cause: DamageSource?) {
//        super.die(cause)
//        health = 20f
//        foodData = FoodMetaData(this)
//        killEntity()
//    }

    override fun getIpAddress() = "127.0.0.1"
}