package nl.kallestruik.dtweaks.fakeplayer

import com.destroystokyo.paper.PaperConfig
import com.destroystokyo.paper.event.player.PlayerInitialSpawnEvent
import com.mojang.authlib.GameProfile
import com.mojang.datafixers.util.Either
import com.mojang.serialization.Dynamic
import io.netty.buffer.Unpooled
import net.minecraft.nbt.CompoundTag
import net.minecraft.nbt.NbtOps
import net.minecraft.network.Connection
import net.minecraft.network.FriendlyByteBuf
import net.minecraft.network.protocol.game.*
import net.minecraft.resources.ResourceKey
import net.minecraft.server.MinecraftServer
import net.minecraft.server.level.*
import net.minecraft.server.level.ChunkHolder.ChunkLoadingFailure
import net.minecraft.server.network.ServerGamePacketListenerImpl
import net.minecraft.server.players.GameProfileCache
import net.minecraft.server.players.PlayerList
import net.minecraft.world.level.ChunkPos
import net.minecraft.world.level.GameRules
import net.minecraft.world.level.Level
import net.minecraft.world.level.biome.BiomeManager
import net.minecraft.world.level.chunk.ChunkAccess
import net.minecraft.world.level.dimension.DimensionType
import nl.kallestruik.dlib.ReflectionUtil
import nl.kallestruik.dtweaks.DTweaks
import org.apache.logging.log4j.Logger
import org.bukkit.Bukkit
import org.bukkit.craftbukkit.v1_18_R1.CraftWorld
import org.bukkit.entity.Player
import org.spigotmc.event.player.PlayerSpawnLocationEvent
import java.util.*
import java.util.concurrent.CompletableFuture
import java.util.function.Consumer

object FakePlayerList {
    fun placeNewPlayer(playerList: PlayerList, server: MinecraftServer, connection: FakeConnection, player: FakePlayer) {
        player.isRealPlayer = true // Paper - Chunk priority

        player.networkManager = connection // Paper

        player.loginTime = System.currentTimeMillis() // Paper

        val gameProfile: GameProfile = player.getGameProfile()
        val userCache: GameProfileCache = server.getProfileCache()
        val optional = userCache[gameProfile.id]
        var s: String? = optional.map { obj: GameProfile -> obj.name }.orElse(gameProfile.name)

        userCache.add(gameProfile)
        val nbtTagCompound: CompoundTag? = playerList.load(player)
        // CraftBukkit start - Better rename detection
        // CraftBukkit start - Better rename detection
        if (nbtTagCompound != null && nbtTagCompound.contains("bukkit")) {
            val bukkit = nbtTagCompound.getCompound("bukkit")
            s = if (bukkit.contains("lastKnownName", 8)) bukkit.getString("lastKnownName") else s
        }
        val lastKnownName = s // Paper

        // CraftBukkit end

        // Paper start - move logic in Entity to here, to use bukkit supplied world UUID.
        // CraftBukkit end

        // Paper start - move logic in Entity to here, to use bukkit supplied world UUID.
        val resourceKey: ResourceKey<Level> = if (nbtTagCompound != null && nbtTagCompound.contains("WorldUUIDMost") && nbtTagCompound.contains("WorldUUIDLeast")) {
                val uid = UUID(nbtTagCompound.getLong("WorldUUIDMost"), nbtTagCompound.getLong("WorldUUIDLeast"))
                val bWorld = Bukkit.getServer().getWorld(uid)
                if (bWorld != null) {
                    (bWorld as CraftWorld).handle.dimension()
                } else {
                    Level.OVERWORLD
                }
            } else if (nbtTagCompound != null) {
                // Vanilla migration support
                // Paper end
                val dataResult = DimensionType.parseLegacy(
                    Dynamic(
                        NbtOps.INSTANCE,
                        nbtTagCompound["Dimension"]
                    )
                )
                dataResult.result().orElse(Level.OVERWORLD)
            } else {
                Level.OVERWORLD
            }

        var worldServer: ServerLevel = server.getLevel(resourceKey) ?: server.overworld()

        if (nbtTagCompound == null) player.fudgeSpawnLocation(worldServer) // Paper - only move to spawn on first login, otherwise, stay where you are....

        player.setLevel(worldServer)


        // Spigot start - spawn location event
        val spawnPlayer: Player = player.bukkitEntity
        val ev: PlayerSpawnLocationEvent =
            PlayerInitialSpawnEvent(spawnPlayer, spawnPlayer.location) // Paper use our duplicate event

        Bukkit.getServer().pluginManager.callEvent(ev)

        val loc = ev.spawnLocation
        worldServer = (loc.world as CraftWorld).handle

        player.spawnIn(worldServer)
        player.gameMode.setLevel(player.level as ServerLevel)
        // Paper start - set raw so we aren't fully joined to the world (not added to chunk or world)
        // Paper start - set raw so we aren't fully joined to the world (not added to chunk or world)
        player.setPosRaw(loc.x, loc.y, loc.z)
        player.setRot(loc.yaw, loc.pitch)
        // Paper end
        // Spigot end

        // CraftBukkit - Moved message to after join
        // PlayerList.LOGGER.info("{}[{}] logged in with entity id {} at ({}, {}, {})", entityplayer.getName().getString(), s1, entityplayer.getId(), entityplayer.getX(), entityplayer.getY(), entityplayer.getZ());
        // Paper end
        // Spigot end

        // CraftBukkit - Moved message to after join
        // PlayerList.LOGGER.info("{}[{}] logged in with entity id {} at ({}, {}, {})", entityplayer.getName().getString(), s1, entityplayer.getId(), entityplayer.getX(), entityplayer.getY(), entityplayer.getZ());
        val worlddata = worldServer.getLevelData()

        player.loadGameTypes(nbtTagCompound)
        val playerconnection = FakePlayerConnection(server, connection, player)
        val gamerules = worldServer.gameRules

        player.bukkitEntity.sendSupportedChannels() // CraftBukkit

        player.stats.markAllDirty()
        player.recipeBook.sendInitialRecipeBook(player)
        playerList.updateEntireScoreboard(worldServer.scoreboard, player)
        server.invalidateStatus()
        // Paper start - async load spawn in chunk
        // Paper start - async load spawn in chunk
        val finalWorldserver = worldServer
        val chunkX = loc.blockX shr 4
        val chunkZ = loc.blockZ shr 4
        val pos = ChunkPos(chunkX, chunkZ)
        val playerChunkMap = worldServer.getChunkSource().chunkMap
        val distanceManager: DistanceManager = playerChunkMap.distanceManager
        distanceManager.addTicketAtLevel(TicketType.LOGIN, pos, 31, pos.toLong())
        worldServer.getChunkSource().markAreaHighPriority(pos, 28, 3) // Paper - Chunk priority

        val postChunkLoadJoin = playerList::class.java.superclass.getDeclaredMethod("postChunkLoadJoin", ServerPlayer::class.java, ServerLevel::class.java, Connection::class.java, ServerGamePacketListenerImpl::class.java, CompoundTag::class.java, String::class.java, String::class.java)
        postChunkLoadJoin.isAccessible = true
        postChunkLoadJoin.invoke(playerList, player, finalWorldserver, connection, playerconnection,
                nbtTagCompound, "127.0.0.1", lastKnownName)


//        worldServer.getChunkSource().getChunkAtAsynchronously(chunkX, chunkZ, true, false)
//            .thenApply { chunk: Either<ChunkAccess, ChunkLoadingFailure> ->  // Paper - Chunk priority
//                val updatingChunk = playerChunkMap.getUpdatingChunkIfPresent(pos.toLong())
//                if (updatingChunk != null) {
//                    return@thenApply updatingChunk.entityTickingChunkFuture
//                } else {
//                    return@thenApply CompletableFuture.completedFuture(
//                        chunk
//                    )
//                }
//            }.thenAccept {
//
//        }
    }
}