package nl.kallestruik.dtweaks.tweaks.miscellaneoustweaks

import nl.kallestruik.dtweaks.DTweaks
import nl.kallestruik.dtweaks.tweaks.ITweak
import org.bukkit.Material
import org.bukkit.NamespacedKey
import org.bukkit.block.Block
import org.bukkit.block.BlockFace
import org.bukkit.event.EventHandler
import org.bukkit.event.HandlerList
import org.bukkit.event.Listener
import org.bukkit.event.block.BlockBreakEvent
import org.bukkit.inventory.*
import org.bukkit.inventory.meta.Damageable
import org.bukkit.persistence.PersistentDataType


class Hammers(
    private val plugin: DTweaks
): ITweak, Listener {
    override fun getIdentifier(): String = "Hammers"
    override fun getCategories(): List<String> = listOf("miscellaneous", "survival")

    val HAMMER_KEY = NamespacedKey(plugin, "hammer")
    val diamondHammerRecipeKey = NamespacedKey(plugin, "diamond_hammer")

    override fun onEnable() {
        plugin.server.pluginManager.registerEvents(this, plugin)

        val diamondHammer = ItemStack(Material.DIAMOND_PICKAXE)
        val diamondHammerMeta = diamondHammer.itemMeta
        diamondHammerMeta.persistentDataContainer.set(HAMMER_KEY, PersistentDataType.SHORT, 1)
        diamondHammer.itemMeta = diamondHammerMeta
        updateHammer(diamondHammer)

        val recipe = ShapedRecipe(diamondHammerRecipeKey, diamondHammer)
        recipe.shape(
            " BD",
            " SB",
            "S  ")
        recipe.setIngredient('B', Material.DIAMOND_BLOCK)
        recipe.setIngredient('D', Material.DIAMOND)
        recipe.setIngredient('S', Material.STICK)

        plugin.server.addRecipe(recipe)
    }

    override fun onDisable() {
        HandlerList.unregisterAll(this)
        plugin.server.removeRecipe(diamondHammerRecipeKey)
    }

    fun updateHammer(hammer: ItemStack) {
        if (!isHammer(hammer))
            return
        val meta = hammer.itemMeta

        meta.setCustomModelData(1)

        hammer.itemMeta = meta
    }

    private fun isHammer(itemStack: ItemStack): Boolean {
        val meta = itemStack.itemMeta ?: return false

        val persistentContainer = meta.persistentDataContainer

        return persistentContainer.get(HAMMER_KEY, PersistentDataType.SHORT) == 1.toShort()
    }

    @EventHandler
    fun onBlockBreak(event: BlockBreakEvent) {
        val hammer = event.player.inventory.itemInMainHand
        if (!isHammer(hammer))
            return

        val player = event.player

        val lastTwoTargetBlocks: List<Block> = player.getLastTwoTargetBlocks(null, 100)
        if (lastTwoTargetBlocks.size != 2 || !lastTwoTargetBlocks[1].type.isOccluding) return

        val targetBlock: Block = lastTwoTargetBlocks[1]
        val adjacentBlock: Block = lastTwoTargetBlocks[0]

        val targetFace = targetBlock.getFace(adjacentBlock)

        val toBreak: MutableList<Block> = mutableListOf()

        when (targetFace) {
            BlockFace.NORTH, BlockFace.SOUTH -> {
                toBreak.add(targetBlock.getRelative(1, 0, 0))
                toBreak.add(targetBlock.getRelative(-1, 0, 0))
                toBreak.add(targetBlock.getRelative(0, 1, 0))
                toBreak.add(targetBlock.getRelative(0, -1, 0))
                toBreak.add(targetBlock.getRelative(1, 1, 0))
                toBreak.add(targetBlock.getRelative(-1, 1, 0))
                toBreak.add(targetBlock.getRelative(1, -1, 0))
                toBreak.add(targetBlock.getRelative(-1, -1, 0))
            }

            BlockFace.EAST, BlockFace.WEST -> {
                toBreak.add(targetBlock.getRelative(0, 1, 0))
                toBreak.add(targetBlock.getRelative(0, -1, 0))
                toBreak.add(targetBlock.getRelative(0, 0, 1))
                toBreak.add(targetBlock.getRelative(0, 0, -1))
                toBreak.add(targetBlock.getRelative(0, 1, 1))
                toBreak.add(targetBlock.getRelative(0, -1, 1))
                toBreak.add(targetBlock.getRelative(0, 1, -1))
                toBreak.add(targetBlock.getRelative(0, -1, -1))
            }

            BlockFace.UP, BlockFace.DOWN -> {
                toBreak.add(targetBlock.getRelative(1, 0, 0))
                toBreak.add(targetBlock.getRelative(-1, 0, 0))
                toBreak.add(targetBlock.getRelative(0, 0, 1))
                toBreak.add(targetBlock.getRelative(0, 0, -1))
                toBreak.add(targetBlock.getRelative(1, 0, 1))
                toBreak.add(targetBlock.getRelative(-1, 0, 1))
                toBreak.add(targetBlock.getRelative(1, 0, -1))
                toBreak.add(targetBlock.getRelative(-1, 0, -1))
            }
            else -> return
        }

        val maxHardness = targetBlock.type.hardness

        for (block in toBreak) {
            if (block.type.hardness > maxHardness)
                continue

            block.breakNaturally(hammer)
        }
    }
}