package nl.kallestruik.dtweaks.tweaks.mobtweaks

import net.minecraft.core.GlobalPos
import net.minecraft.world.entity.ai.memory.MemoryModuleType
import net.minecraft.world.entity.schedule.Activity
import nl.kallestruik.dtweaks.tweaks.ITweak
import org.bukkit.craftbukkit.v1_18_R1.entity.CraftVillager
import org.bukkit.entity.Villager
import org.bukkit.event.EventHandler
import org.bukkit.event.HandlerList
import org.bukkit.event.Listener
import org.bukkit.event.player.PlayerInteractEntityEvent
import org.bukkit.plugin.java.JavaPlugin
import java.util.concurrent.atomic.AtomicReference

class VillagerInfo(
    private val plugin: JavaPlugin
): ITweak, Listener {
    override fun getIdentifier(): String = "VillagerInfo"
    override fun getCategories(): List<String> = listOf("mobs")

    override fun onEnable() {
        plugin.server.pluginManager.registerEvents(this, plugin)
    }

    override fun onDisable() {
        HandlerList.unregisterAll(this)
    }

    @EventHandler
    fun onInteractEvent(event: PlayerInteractEntityEvent) {
        if (!event.player.isSneaking) return
        val clicked = event.rightClicked as? Villager ?: return
        val eVillager: net.minecraft.world.entity.npc.Villager = (clicked as CraftVillager).handle
        val homePos: AtomicReference<GlobalPos?> = AtomicReference(null)
        eVillager.brain.getMemory(MemoryModuleType.HOME)
            .ifPresent { newValue -> homePos.set(newValue) }
        val workPos: AtomicReference<GlobalPos?> = AtomicReference(null)
        eVillager.brain.getMemory(MemoryModuleType.JOB_SITE).ifPresent { newValue ->
            workPos.set(
                newValue
            )
        }
        val meetingPos: AtomicReference<GlobalPos?> = AtomicReference(null)
        eVillager.brain.getMemory(MemoryModuleType.MEETING_POINT).ifPresent { newValue ->
            meetingPos.set(
                newValue
            )
        }
        val lastWork: AtomicReference<Long?> = AtomicReference(null)
        eVillager.brain.getMemory(MemoryModuleType.LAST_WORKED_AT_POI).ifPresent { newValue ->
            lastWork.set(
                newValue
            )
        }
        val lastSleep: AtomicReference<Long?> = AtomicReference(null)
        eVillager.brain.getMemory(MemoryModuleType.LAST_SLEPT).ifPresent { newValue ->
            lastSleep.set(
                newValue
            )
        }
        val seenIGRecently: AtomicReference<Boolean?> = AtomicReference(null)
        eVillager.brain.getMemory(MemoryModuleType.GOLEM_DETECTED_RECENTLY).ifPresent { newValue ->
            seenIGRecently.set(
                newValue
            )
        }

        val isPanicking: Boolean = eVillager.brain.isActive(Activity.PANIC)
        val player = event.player

        player.sendMessage("=====Villager Info=====")
        player.sendMessage("Type: " + clicked.villagerType)
        player.sendMessage("Profession: " + clicked.profession)
        player.sendMessage("Level: " + clicked.villagerLevel + "(" + clicked.villagerExperience + " xp)")
        player.sendMessage("Panic: $isPanicking")

        if (homePos.get() != null) player.sendMessage(
            java.lang.String.format(
                "Home: %s, %s, %s",
                homePos.get()?.pos()?.x,
                homePos.get()?.pos()?.y,
                homePos.get()?.pos()?.z
            )
        )

        if (workPos.get() != null) player.sendMessage(
            java.lang.String.format(
                "Work: %s, %s, %s",
                workPos.get()?.pos()?.x,
                workPos.get()?.pos()?.y,
                workPos.get()?.pos()?.z
            )
        )

        if (homePos.get() != null) player.sendMessage(
            java.lang.String.format(
                "Meeting point: %s, %s, %s",
                meetingPos.get()?.pos()?.x,
                meetingPos.get()?.pos()?.y,
                meetingPos.get()?.pos()?.z
            )
        )

        if (lastSleep.get() != null)
            player.sendMessage("Last slept: ${lastSleep.get()}")

        if (seenIGRecently.get() != null)
            player.sendMessage("Seen iron golem Recently: ${seenIGRecently.get()}")
    }
}